@@ -26,6 +26,9 @@ error = compare x y /= GT ==> x <= y
 error = compare x y == LT ==> x < y
 error = compare x y /= LT ==> x >= y
 error = compare x y == GT ==> x > y
+error = compare (f x) (f y) ==> Data.Ord.comparing f x y
+error = on compare f ==> Data.Ord.comparing f
+
 
 -- READ/SHOW
 
@@ -74,6 +77,9 @@ error = (\(_,y) -> y) ==> snd
 error = (\(x,_) -> x) ==> fst
 error = (\x y-> f (x,y)) ==> curry f where _ = notIn [x,y] f
 error = (\(x,y) -> f x y) ==> uncurry f where _ = notIn [x,y] f
+warn  = (\x -> f x y) ==> flip f y where _ = notIn x [f,y]
+error "Redundant id" = id x ==> x
+error "Redundant const" = const x y ==> x
 
 -- BOOL
 
@@ -93,31 +99,38 @@ error "Use if" = case a of {False -> f; _ -> t} ==> if a then t else f
 
 error = id *** g ==> second g
 error = f *** id ==> first f
-warn  = (\(x,y) -> (f x, g y)) ==> f *** g where _ = notIn [x,y] [f,g]
-warn  = (\x -> (f x, g x)) ==> f &&& g where _ = notIn x [f,g]
-warn  = (\(x,y) -> (f x,y)) ==> first f where _ = notIn [x,y] f
-warn  = (\(x,y) -> (x,f y)) ==> second f where _ = notIn [x,y] f
+warn  = (\(x,y) -> (f x, g y)) ==> f Control.Arrow.*** g where _ = notIn [x,y] [f,g]
+warn  = (\x -> (f x, g x)) ==> f Control.Arrow.&&& g where _ = notIn x [f,g]
+warn  = (\(x,y) -> (f x,y)) ==> Control.Arrow.first f where _ = notIn [x,y] f
+warn  = (\(x,y) -> (x,f y)) ==> Control.Arrow.second f where _ = notIn [x,y] f
+
+-- FUNCTOR
+
+error "Functor law" = fmap f (fmap g x) ==> fmap (f . g) x
+error "Functor law" = fmap id ==> id
 
 -- MONAD
 
 error "Monad law, left identity" = return a >>= f ==> f a
 error "Monad law, right identity" = m >>= return ==> m
-warn  = m >>= return . f ==> liftM f m
-error = (if x then y else return ()) ==> when x $ _noParen_ y
+warn  = m >>= return . f ==> fmap f m
+error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y
+error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y
 error = sequence (map f x) ==> mapM f x
 error = sequence_ (map f x) ==> mapM_ f x
-warn  = flip mapM ==> forM
-warn  = flip mapM_ ==> forM_
+warn  = flip mapM ==> Control.Monad.forM
+warn  = flip mapM_ ==> Control.Monad.forM_
 error = when (not x) ==> unless x
-error = x >>= id ==> join x
+error = x >>= id ==> Control.Monad.join x
+error = liftM f (liftM g x) ==> liftM (f . g) x
 
 -- MONAD LIST
 
-error = liftM unzip (mapM f x) ==> mapAndUnzipM f x
-error = sequence (zipWith f x y) ==> zipWithM f x y
-error = sequence_ (zipWith f x y) ==> zipWithM_ f x y
-error = sequence (replicate n x) ==> replicateM n x
-error = sequence_ (replicate n x) ==> replicateM_ n x
+error = liftM unzip (mapM f x) ==> Control.Monad.mapAndUnzipM f x
+error = sequence (zipWith f x y) ==> Control.Monad.zipWithM f x y
+error = sequence_ (zipWith f x y) ==> Control.Monad.zipWithM_ f x y
+error = sequence (replicate n x) ==> Control.Monad.replicateM n x
+error = sequence_ (replicate n x) ==> Control.Monad.replicateM_ n x
 error = mapM f (map g x) ==> mapM (f . g) x
 error = mapM_ f (map g x) ==> mapM_ (f . g) x
 
@@ -132,9 +145,9 @@ error "Redundant $!" = id $! x ==> x
 
 -- MAYBE
 
-error = maybe x id  ==> fromMaybe x
-error = maybe False (const True) ==> isJust
-error = maybe True (const False) ==> isNothing
+error = maybe x id  ==> Data.Maybe.fromMaybe x
+error = maybe False (const True) ==> Data.Maybe.isJust
+error = maybe True (const False) ==> Data.Maybe.isNothing
 error = catMaybes (map f x) ==> mapMaybe f x
 
 -- MATHS
@@ -190,50 +203,65 @@ error "Evaluate" = id x ==> x
 
 -- COMPLEX
 
-error "Use isPrefixOf" = (take i s == t) ==> _eval_ ((i == length t) && (t `isPrefixOf` s))
+error "Use isPrefixOf" = (take i s == t) ==> _eval_ ((i == length t) && (t `Data.List.isPrefixOf` s))
     where _ = (isList t || isLit t) && isLit i
 
 warn  = (do a <- f; g a) ==> f >>= g
-    where _ = isAtom f || isApp f
+    where _ = (isAtom f || isApp f) && notIn a g
 
 
 
 {-
 <TEST>
-yes = concat . map f where res = concatMap f
-yes = foo . bar . concat . map f . baz . bar where res = concatMap f . baz . bar
-yes = map f (map g x) where res = map (f . g) x
-yes = concat.map (\x->if x==e then l' else [x]) where res = concatMap (\x->if x==e then l' else [x])
-yes = f x where f x = concat . map head ; res = concatMap head
-yes = concat . map f . g where res = concatMap f . g
-yes = concat $ map f x where res = concatMap f x
-yes = "test" ++ concatMap (' ':) ["of","this"] where res = unwords ("test":["of","this"])
-yes = concat . intersperse " " where res = unwords
-yes = if f a then True else b where res = f a || b
-yes = not (a == b) where res = a /= b
-yes = not (a /= b) where res = a == b
-yes = if a then 1 else if b then 1 else 2 where res = if a || b then 1 else 2
+yes = concat . map f -- concatMap f
+yes = foo . bar . concat . map f . baz . bar -- concatMap f . baz . bar
+yes = map f (map g x) -- map (f . g) x
+yes = concat.map (\x->if x==e then l' else [x]) -- concatMap (\x->if x==e then l' else [x])
+yes = f x where f x = concat . map head -- concatMap head
+yes = concat . map f . g -- concatMap f . g
+yes = concat $ map f x -- concatMap f x
+yes = "test" ++ concatMap (' ':) ["of","this"] -- unwords ("test":["of","this"])
+yes = concat . intersperse " " -- unwords
+yes = if f a then True else b -- f a || b
+yes = not (a == b) -- a /= b
+yes = not (a /= b) -- a == b
+yes = if a then 1 else if b then 1 else 2 -- if a || b then 1 else 2
 no  = if a then 1 else if b then 3 else 2
-yes = a >>= return . id where res = liftM id a
-yes = (x !! 0) + (x !! 2) where res = head x
-yes = if b < 42 then [a] else [] where res = [a | b < 42]
-yes = take 5 (foo xs) == "hello" where res = "hello" `isPrefixOf` foo xs
+yes = a >>= return . id -- fmap id a
+yes = (x !! 0) + (x !! 2) -- head x
+yes = if b < 42 then [a] else [] -- [a | b < 42]
+yes = take 5 (foo xs) == "hello" -- "hello" `Data.List.isPrefixOf` foo xs
 no  = take n (foo xs) == "hello"
-yes = head (reverse xs) where res = last xs
-yes = reverse xs `isPrefixOf` reverse ys where res = isSuffixOf xs ys
+yes = head (reverse xs) -- last xs
+yes = reverse xs `isPrefixOf` reverse ys -- isSuffixOf xs ys
 no = putStrLn $ show (length xs) ++ "Test"
-yes = do line <- getLine; putStrLn line where res = getLine >>= putStrLn 
-yes = ftable ++ map (\ (c, x) -> (toUpper c, urlEncode x)) ftable where res = toUpper *** urlEncode
-yes = map (\(a,b) -> a) xs where res = fst
-yes = map (\(a,_) -> a) xs where res = fst
-yes = readFile $ args !! 0 where res = head args
-yes = if Debug `elem` opts then ["--debug"] else [] where res = ["--debug" | Debug `elem` opts]
-yes = if nullPS s then return False else if headPS s /= '\n' then return False else alter_input tailPS >> return True
-    where res = if nullPS s || (headPS s /= '\n') then return False else alter_input tailPS >> return True
-yes = if foo then do stuff; moreStuff; lastOfTheStuff else return ()
-    where res = when foo $ do stuff ; moreStuff ; lastOfTheStuff
-yes = foo $ \(a, b) -> (a, y + b) where res = second ((+) y)
+yes = do line <- getLine; putStrLn line -- getLine >>= putStrLn 
+yes = ftable ++ map (\ (c, x) -> (toUpper c, urlEncode x)) ftable -- toUpper Control.Arrow.*** urlEncode
+yes = map (\(a,b) -> a) xs -- fst
+yes = map (\(a,_) -> a) xs -- fst
+yes = readFile $ args !! 0 -- head args
+yes = if Debug `elem` opts then ["--debug"] else [] -- ["--debug" | Debug `elem` opts]
+yes = if nullPS s then return False else if headPS s /= '\n' then return False else alter_input tailPS >> return True \
+    -- if nullPS s || (headPS s /= '\n') then return False else alter_input tailPS >> return True
+yes = if foo then do stuff; moreStuff; lastOfTheStuff else return () \
+    -- Control.Monad.when foo $ do stuff ; moreStuff ; lastOfTheStuff
+yes = foo $ \(a, b) -> (a, y + b) -- Control.Arrow.second ((+) y)
 no  = foo $ \(a, b) -> (a, a + b)
-yes = map (uncurry (+)) $ zip [1 .. 5] [6 .. 10] where res = zipWith (+) [1 .. 5] [6 .. 10]
+yes = map (uncurry (+)) $ zip [1 .. 5] [6 .. 10] -- zipWith (+) [1 .. 5] [6 .. 10]
+no = do iter <- textBufferGetTextIter tb ; textBufferSelectRange tb iter iter
+no = flip f x $ \y -> y*y+y
+no = \x -> f x (g x)
+yes = \x -> f x (g y) -- flip f (g y)
+
+import Prelude \
+yes = flip mapM -- Control.Monad.forM
+import Control.Monad \
+yes = flip mapM -- forM
+import Control.Monad(forM) \
+yes = flip mapM -- forM
+import Control.Monad(forM_) \
+yes = flip mapM -- Control.Monad.forM
+import qualified Control.Monad \
+yes = flip mapM -- Control.Monad.forM
 </TEST>
 -}
