@@ -4,4 +4,4 @@ module HLint.Generalise where
 warn = concatMap ==> (>>=)
 warn = liftM ==> fmap
 warn = map ==> fmap
-warn = a ++ b ==> a `mappend` b
+warn = a ++ b ==> a `Data.Monoid.mappend` b
