@@ -5,7 +5,7 @@ module HSE.All(
     module HSE.Bracket, module HSE.Match,
     module HSE.Generics,
     module HSE.NameMatch,
-    parseFile, parseString, fromParseResult
+    ParseFlags(..), parseFlags, parseFile, parseString, fromParseResult
     ) where
 
 import Language.Haskell.Exts hiding (parse, parseFile, paren, fromParseResult)
@@ -20,27 +20,34 @@ import HSE.NameMatch
 import Util
 import Data.Char
 import Data.List
+import Language.Preprocessor.Cpphs
+
+
+data ParseFlags = ParseFlags
+    {cpphs :: Maybe CpphsOptions
+    ,implies :: Bool
+    }
+
+parseFlags :: ParseFlags
+parseFlags = ParseFlags Nothing False
 
 
 -- | Parse a Haskell module
-parseString :: FilePath -> String -> ParseResult Module
-parseString file = parseFileContentsWithMode mode . unlines . map f . lines
+parseString :: ParseFlags -> FilePath -> String -> ParseResult Module
+parseString flags file = parseFileContentsWithMode mode . maybe id (`runCpphs` file) (cpphs flags)
     where
-        f x | "#" `isPrefixOf` dropWhile isSpace x = ""
-            | otherwise = x
-    
         mode = defaultParseMode
             {parseFilename = file
             ,extensions = extension
-            ,fixities = infix_ (-1) ["==>"] ++ baseFixities
+            ,fixities = concat [infix_ (-1) ["==>"] | implies flags] ++ baseFixities
             }
 
 
 -- | On failure returns an empty module and prints to the console
-parseFile :: FilePath -> IO (ParseResult Module)
-parseFile file = do
+parseFile :: ParseFlags -> FilePath -> IO (ParseResult Module)
+parseFile flags file = do
     src <- readFile file
-    return $ parseString file src
+    return $ parseString flags file src
 
 
 -- | TODO: Use the fromParseResult in HSE once it gives source location
@@ -67,4 +74,3 @@ extension =
     -- NOT: TransformListComp - steals the group keyword
     -- NOT: XmlSyntax, RegularPatterns - steals a-b
     ]
-
