@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, ViewPatterns, MultiParamTypeClasses #-}
+{-# LANGUAGE ViewPatterns, MultiParamTypeClasses #-}
 
 module HSE.Match where
 
@@ -27,15 +27,8 @@ instance View Exp App1 where
     view _ = NoApp1
 
 
-data Infix = NoInfix | Infix Exp Exp Exp deriving Show
-
-instance View Exp Infix where
-    view (fromParen -> InfixApp a b c) = Infix a (opExp b) c
-    view _ = NoInfix
-
-
 (~=) :: Exp -> String -> Bool
-(~=) x y = fromNamed x == y
+(~=) = (==) . fromNamed
 
 
 -- | fromNamed will return "" when it cannot be represented
@@ -98,6 +91,12 @@ instance Named TyVarBind where
     toNamed = UnkindedVar . toNamed
 
 
+instance Named QOp where
+    fromNamed (QVarOp x) = fromNamed x
+    fromNamed (QConOp x) = fromNamed x
+    toNamed x | isCon x = QConOp $ toNamed x
+              | otherwise = QVarOp $ toNamed x
+
 instance Named Decl where
     fromNamed (TypeDecl _ name _ _) = fromNamed name
     fromNamed (DataDecl _ _ _ name _ _ _) = fromNamed name
