@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, ViewPatterns, MultiParamTypeClasses #-}
+{-# LANGUAGE PatternGuards #-}
 
 module HSE.Util where
 
@@ -27,6 +27,9 @@ moduleName (Module _ (ModuleName x) _ _ _ _ _) = x
 moduleImports :: Module -> [ImportDecl]
 moduleImports (Module _ _ _ _ _ x _) = x
 
+modulePragmas :: Module -> [OptionPragma]
+modulePragmas (Module _ _ x _ _ _ _) = x
+
 isChar :: Exp -> Bool
 isChar (Lit (Char _)) = True
 isChar _ = False
@@ -54,17 +57,22 @@ isApp App{} = True; isApp _ = False
 isInfixApp InfixApp{} = True; isInfixApp _ = False
 isAnyApp x = isApp x || isInfixApp x
 isParen Paren{} = True; isParen _ = False
-
+isMDo MDo{} = True; isMDo _ = False
+isBoxed Boxed{} = True; isBoxed _ = False
+isDerivDecl DerivDecl{} = True; isDerivDecl _ = False
+isFunDep FunDep{} = True; isFunDep _ = False
+isPBangPat PBangPat{} = True; isPBangPat _ = False
+isPExplTypeArg PExplTypeArg{} = True; isPExplTypeArg _ = False
+isPFieldPun PFieldPun{} = True; isPFieldPun _ = False
+isPFieldWildcard PFieldWildcard{} = True; isPFieldWildcard _ = False
+isPViewPat PViewPat{} = True; isPViewPat _ = False
+isParComp ParComp{} = True; isParComp _ = False
+isPatTypeSig PatTypeSig{} = True; isPatTypeSig _ = False
+isQuasiQuote QuasiQuote{} = True; isQuasiQuote _ = False
 
 ---------------------------------------------------------------------
 -- HSE FUNCTIONS
 
-
-instance Eq Module where
-    Module x1 x2 x3 x4 x5 x6 x7 == Module y1 y2 y3 y4 y5 y6 y7 =
-        x1 == y1 && x2 == y2 && x3 == y3 && x4 == y4 && x5 == y5 && x6 == y6 && x7 == y7
-
-
 -- pick a variable that is not being used
 freeVar :: Data a => a -> String
 freeVar x = head $ allVars \\ concat [[y, drop 1 y] | Ident y <- universeBi x]
