@@ -12,20 +12,31 @@
     -- don't eta reduce func a b c = .... (g b) (g c) to (g b) . g, looks ugly
 
 <TEST>
-yes = 0 where f a = \x -> x + x ; res = "f a x = x + x"
-yes = 0 where h a = f (g a ==) ; res = "h = f . (==) . g"
-yes a = foo (\x -> True) where res = const True
-no = foo (\x -> map f [])
-yes = 0 where f x = y x ; res = "f = y"
-no mr = y mr
-yes = 0 where f x = g $ f $ map head x ; res = "f = g . f . map head"
-no z x y = f (g x) (g y)
-no = 0 where f x y = f (g x) (g y) ; res = "f = f `on` g"
-yes = 0 where f x y = g x == g y ; res = "f = (==) `on` g"
-a + b = foo a b where res = "(+) = foo"
-type Yes a = Foo Char a
-yes = foo (\x -> sum x) where res = sum
-yes = foo (\x l -> sum x x l) where res = \x -> sum x x
+f a = \x -> x + x -- f a x = x + x
+h a = f (g a ==) -- h = f . (==) . g
+test a = foo (\x -> True) -- const True
+test = foo (\x -> map f [])
+test = 0 where f x = y x -- f = y
+test mr = y mr
+test = 0 where f x = g $ f $ map head x -- f = g . f . map head
+test z x y = f (g x) (g y)
+f x y = f (g x) (g y)
+f x y = g x == g y -- f = (==) `on` g
+a + b = foo a b -- (+) = foo
+h a = f ((++) a) a -- (a ++)
+h a = flip f x (y z) -- f (y z) x
+h a = flip f x $ y z
+type Test a = Foo Char a
+type Test a = Foo a Char a
+type Test (a :: * -> *) = Foo Char a
+yes = foo (\x -> sum x) -- sum
+yes = foo (\x l -> sum x x l) -- \x -> sum x x
+test = foo (\x -> y == x) -- (y ==)
+test = foo (\x -> x == g y) -- (== g y)
+test = foo (\x -> g x == x)
+cp i = show (i - 1)
+cp i = show (i + 1)
+test c = 1 # 2 # c
 </TEST>
 -}
 
@@ -34,29 +45,32 @@ module Hint.Lambda where
 
 import HSE.All
 import Type
-import Control.Arrow
 import Control.Monad
 import Data.Generics.PlateData
 import Data.Maybe
 
 
-lambdaHint :: Hint
+lambdaHint :: DeclHint
 lambdaHint _ x@TypeDecl{} = lambdaType x
-lambdaHint _ x = concatMap lambdaExp (universeBi x) ++ concatMap lambdaDecl (universe x)
+lambdaHint _ x = concatMap (uncurry lambdaExp) (universeExp nullSrcLoc x) ++ concatMap lambdaDecl (universe x)
 
 
-lambdaExp :: Exp -> [Idea]
-lambdaExp o@(Lambda loc [v] y) | isAtom y, Just x <- f v, x `notElem` universeBi y =
+lambdaExp :: SrcLoc -> Exp -> [Idea]
+lambdaExp _ o@(Lambda loc [v] y) | isAtom y, Just x <- f v, x `notElem` universeBi y =
         [warn "Use const" loc o res]
     where
         f (PVar x) = Just x
         f PWildCard = Just $ Ident "_"
         f _ = Nothing
         res = App (toNamed "const") y
-lambdaExp o@(Lambda loc vs x) | length vs /= length vs2 =
+lambdaExp _ o@(Lambda loc vs x) | length vs /= length vs2 =
         [warn "Eta reduce" loc o $ if null vs2 then x2 else Lambda loc vs2 x2]
     where (vs2,x2) = etaReduces vs x
-lambdaExp _ = []
+lambdaExp loc o@(Paren (App (Var x@(UnQual (Symbol _))) y)) | isAtom y =
+        [warn "Operator rotate" loc o $ LeftSection y (QVarOp x)]
+lambdaExp loc o@(App (App (App flp x) y) z) | flp ~= "flip" =
+        [idea Error "Redundant flip" loc o $ App (App x z) y]
+lambdaExp _ _ = []
 
 
 lambdaDecl :: Decl -> [Idea]
@@ -69,7 +83,7 @@ lambdaDef :: Match -> [Idea]
 lambdaDef o@(Match loc name pats typ (UnGuardedRhs bod) (BDecls []))
     | Lambda loc vs y <- bod = [warn "Redundant lambda" loc o $ reform (pats++vs) y]
     | [PVar x, PVar y] <- pats, Just (f,g) <- useOn x y bod =
-              [warn "Use on" loc o $ reform [] (ensureBracket1 $ InfixApp f (QVarOp $ UnQual $ Ident "on") g)]
+              [warn "Use on" loc o $ reform [] (ensureBracket1 $ InfixApp f (toNamed "on") g)]
     | (p2,y) <- etaReduces pats bod, length p2 /= length pats = [warn "Eta reduce" loc o $ reform p2 y]
     | otherwise = []
         where reform pats2 bod2 = Match loc name pats2 typ (UnGuardedRhs bod2) (BDecls [])
@@ -91,9 +105,13 @@ etaReduces ps x | ps /= [], PVar p <- last ps, p /= Ident "mr", Just y <- etaRed
 
 etaReduce :: Name -> Exp -> Maybe Exp
 etaReduce x (App y (Var (UnQual z))) | x == z && x `notElem` universeBi y = Just y
+etaReduce x (InfixApp y op z) | f y z = Just $ RightSection op z
+                              | f z y = Just $ LeftSection y op
+    where f y z = op `notElem` map (toNamed . return) "+-" &&
+                  all (not . isInfixApp) [y,z] && var x == y && x `notElem` universeBi z
 etaReduce x (App y z) | not (uglyEta y z) && x `notElem` universeBi y = do
     z2 <- etaReduce x z
-    return $ InfixApp y (QVarOp $ UnQual $ Symbol ".") z2
+    return $ InfixApp y (toNamed ".") z2
 etaReduce x (view -> App2 dollar y z) | dollar ~= "$" = etaReduce x (App y z)
 etaReduce x (LeftSection y op) = etaReduce x $ App (opExp op) y
 etaReduce x y | isParen y = etaReduce x (fromParen y)
@@ -107,13 +125,23 @@ uglyEta _ _ = False
 
 
 
+-- NOTE: Never perform lambda reduction on types
+-- > type Foo a = Bar a
+-- can be reduced for type Foo = Bar, only if Bar is
+-- a data type, not a type alias - but we can't know
+-- which it is
 lambdaType :: Decl -> [Idea]
-lambdaType o@(TypeDecl src name args typ) = [warn "Type eta reduce" src o t2 | i /= 0]
+lambdaType o@(TypeDecl src name args typ) = []
+{-
+        [warn "Type eta reduce" src o t2 | i /= 0]
     where
         (i,t) = f (reverse args) typ
         t2 = TypeDecl src name (take (length args - i) args) t
     
         -- return the number you managed to delete
         f :: [TyVarBind] -> Type -> (Int, Type)
-        f (x:xs) (TyApp t1 (TyVar v)) | fromNamed v == fromNamed x = first (+1) $ f xs t1
+        f (UnkindedVar x:xs) (TyApp t1 (TyVar v))
+            | fromNamed v == fromNamed x && x `notElem` universeBi t1
+            = first (+1) $ f xs t1
         f _ t = (0,t)
+-}
