@@ -13,11 +13,11 @@ foldl f z (x:xs) = foldl f (f z x) xs
 
 {-
 <TEST>
-yes = 0 where f (x:xs) = negate x + f xs ; f [] = 0 ; res = "f xs = foldr ((+) . negate) 0 xs"
-yes = 0 where f (x:xs) = x + 1 : f xs ; f [] = [] ; res = "f xs = map (+ 1) xs"
-yes = 0 where f z (x:xs) = f (z*x) xs ; f z [] = z ; res = "f z xs = foldl (*) z xs"
-yes = 0 where f a (x:xs) b = x + a + b : f a xs b ; f a [] b = [] ; res = "f a xs b = map (\\ x -> x + a + b) xs"
-yes = 0 where f [] a = return a ; f (x:xs) a = a + x >>= \fax -> f xs fax ; res = "f xs a = foldM (+) a xs"
+f (x:xs) = negate x + f xs ; f [] = 0 -- f xs = foldr ((+) . negate) 0 xs
+f (x:xs) = x + 1 : f xs ; f [] = [] -- f xs = map (+ 1) xs
+f z (x:xs) = f (z*x) xs ; f z [] = z -- f z xs = foldl (*) z xs
+f a (x:xs) b = x + a + b : f a xs b ; f a [] b = [] -- f a xs b = map (\ x -> x + a + b) xs
+f [] a = return a ; f (x:xs) a = a + x >>= \fax -> f xs fax -- f xs a = foldM (+) a xs
 </TEST>
 -}
 
@@ -35,7 +35,7 @@ import Control.Monad
 import Data.Generics.PlateData
 
 
-listRecHint :: Hint
+listRecHint :: DeclHint
 listRecHint _ = concatMap f . universe
     where
         f o = maybeToList $ do
@@ -181,7 +181,7 @@ lambda xs (Lambda s (PVar v:vs) x) = lambda (xs++[v]) (Lambda s vs x)
 lambda xs (Lambda _ [] x) = lambda xs x
 lambda [x] (App a b) | Var (UnQual x) == b = a
 lambda [x] (App a (Paren (App b c)))
-    | isAtom a && isAtom b && Var (UnQual x) == c = InfixApp a (QVarOp $ UnQual $ Symbol ".") b
+    | isAtom a && isAtom b && Var (UnQual x) == c = InfixApp a (toNamed ".") b
 lambda [x] (InfixApp a op b)
     | a == Var (UnQual x) = RightSection op b
     | b == Var (UnQual x) = LeftSection a op
