@@ -16,19 +16,20 @@ import Type
 import HSE.All
 import Control.Monad
 import Data.Function
+import Util
 import HSE.Evaluate(evaluate)
 
 
 ---------------------------------------------------------------------
 -- PERFORM MATCHING
 
-readMatch :: [Setting] -> Hint
+readMatch :: [Setting] -> DeclHint
 readMatch = findIdeas . filter isMatchExp
 
 
 findIdeas :: [Setting] -> NameMatch -> Decl -> [Idea]
 findIdeas matches nm decl =
-  [ idea (rank m) (hint m) loc x y
+  [ idea (rankS m) (hintS m) loc x y
   | (loc, x) <- universeExp nullSrcLoc decl, not $ isParen x
   , m <- matches, Just y <- [matchIdea nm m x]]
 
@@ -38,7 +39,7 @@ matchIdea nm MatchExp{lhs=lhs,rhs=rhs,side=side} x = do
     u <- unify nm lhs x
     u <- check u
     guard $ checkSide side u
-    return $ dotContract $ performEval $ subst u rhs
+    return $ unqualify nm $ dotContract $ performEval $ subst u rhs
 
 
 -- unify a b = c, a[c] = b
@@ -74,8 +75,8 @@ concatZipWithM f xs = liftM concat . zipWithM f xs
 
 -- check the unification is valid
 check :: [(String,Exp)] -> Maybe [(String,Exp)]
-check = mapM f . groupBy ((==) `on` fst) . sortBy (compare `on` fst)
-    where f xs = if length (nub xs) == 1 then Just (head xs) else Nothing
+check = mapM f . groupSortFst
+    where f (x,ys) = if length (nub ys) == 1 then Just (x,head ys) else Nothing
 
 
 checkSide :: Maybe Exp -> [(String,Exp)] -> Bool
@@ -125,10 +126,19 @@ dotContract x = fromMaybe x (f x)
     where
         f x | isParen x = f $ fromParen x
         f (App x y) | "?" <- fromNamed y = Just x
-                    | Just z <- f y = Just $ InfixApp x (QVarOp $ UnQual $ Symbol ".") z
+                    | Just z <- f y = Just $ InfixApp x (toNamed ".") z
         f _ = Nothing
 
 -- if it has _eval_ do evaluation on it
 performEval :: Exp -> Exp
 performEval (App e x) | e ~= "_eval_" = evaluate x
 performEval x = x
+
+
+-- contract Data.List.foo ==> foo, if Data.List is loaded
+unqualify :: NameMatch -> Exp -> Exp
+unqualify nm = transformBi f
+    where
+        f (Qual mod x) | nm (Qual mod x) (UnQual x) = UnQual x
+        f x = x
+
