@@ -1,5 +1,3 @@
-{-# LANGUAGE ViewPatterns, PatternGuards #-}
-
 {-
     Suggest the use of camelCase
 
@@ -10,12 +8,12 @@
     Also disallow prop_ as it's a standard QuickCheck idiom
 
 <TEST>
-data Yes = Foo | Bar'Test
-data Yes = Bar | Test_Bar
+data Yes = Foo | Bar'Test -- data Yes = Foo | BarTest
+data Yes = Bar | Test_Bar -- data Yes = Bar | TestBar
 data No = a :::: b
-data Yes = Foo {bar_cap :: Int}
+data Yes = Foo {bar_cap :: Int} -- data Yes = Foo{barCap :: Int}
 data No = FOO | BarBAR | BarBBar
-yes_foo = yes_foo + yes_foo where res = "yesFoo = ..."
+yes_foo = yes_foo + yes_foo -- yesFoo = ...
 no = 1 where yes_foo = 2
 a -== b = 1
 </TEST>
@@ -31,7 +29,7 @@ import Data.Char
 import Data.Maybe
 
 
-namingHint :: Hint
+namingHint :: DeclHint
 namingHint _ x = [warn "Use camelCase" (declSrcLoc x) x2 (replaceNames res x2) | not $ null res]
     where res = [(n,y) | n <- nub $ getNames x, Just y <- [suggestName n]]
           x2 = shorten x
@@ -43,7 +41,7 @@ shorten x = case x of
     PatBind a b c d _ -> f (PatBind a b c) d
     x -> x
     where
-        dots = Var $ UnQual $ Ident "..."
+        dots = Var $ UnQual $ Ident "..." -- Must be an Ident, not a Symbol
         f cont (UnGuardedRhs _) = cont (UnGuardedRhs dots) (BDecls [])
         f cont (GuardedRhss _) = cont (GuardedRhss [GuardedRhs nullSrcLoc [Qualifier dots] dots]) (BDecls [])
 
